"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Actions = exports.ActionType = void 0;
const api_1 = require("./api");
const utils_1 = require("@ztimson/utils");
const rxjs_1 = require("rxjs");
var ActionType;
(function (ActionType) {
    ActionType[ActionType["CRON"] = 0] = "CRON";
    ActionType[ActionType["EVENT"] = 1] = "EVENT";
    ActionType[ActionType["DELETE"] = 2] = "DELETE";
    ActionType[ActionType["GET"] = 3] = "GET";
    ActionType[ActionType["PATCH"] = 4] = "PATCH";
    ActionType[ActionType["POST"] = 5] = "POST";
    ActionType[ActionType["PUT"] = 6] = "PUT";
})(ActionType || (exports.ActionType = ActionType = {}));
class Actions extends utils_1.TypedEmitter {
    api;
    $cache = new rxjs_1.BehaviorSubject([]);
    get cache() { return this.$cache.value; }
    set cache(val) { this.$cache.next(val); }
    constructor(api) {
        super();
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
    }
    delete(id) {
        return this.api.request({ url: `/api/actions/${id}`, method: 'DELETE' }).then(() => {
            this.cache = this.cache.filter(a => a._id != id);
            this.emit('DELETE', id);
        });
    }
    list() {
        return this.api.request({ url: `/api/actions` }).then(resp => {
            if (resp.data)
                this.cache = resp.data;
            this.emit('LIST', resp.data || []);
            return resp.data;
        });
    }
    read(id, reload = false) {
        const cached = this.cache.find(a => a._id == id);
        if (!reload && cached)
            return Promise.resolve(cached);
        return this.api.request({ url: `/api/actions/${id}` }).then(action => {
            if (action.data)
                this.cache = this.cache.filter(a => a._id != id).concat([action.data]);
            this.emit('READ', action.data);
            return action.data;
        });
    }
    run(path, opts = {}) {
        return this.api.request({ url: (`/api/actions/run/` + path).replaceAll('//', '/'), ...opts })
            .then(resp => resp.data);
    }
    runById(action, opts = {}) {
        const id = typeof action == 'string' ? action : action._id;
        return this.api.request({ url: '/api/actions/run-by-id/' + id, method: 'POST', ...opts })
            .then(resp => resp.data);
    }
    update(action) {
        return this.api.request({
            url: `/api/actions${action._id ? `/${action._id}` : ''}`,
            method: 'POST',
            body: action
        }).then(action => {
            if (action.data)
                this.cache = this.cache.filter(a => a._id != action.data?._id).concat([action.data]);
            this.emit('UPDATE', action.data);
            return action.data;
        });
    }
}
exports.Actions = Actions;
