"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Payments = void 0;
const utils_1 = require("@ztimson/utils");
const api_1 = require("./api");
class Payments extends utils_1.TypedEmitter {
    api;
    stripe;
    constructor(api, secret) {
        super();
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
        const setup = (retry = 1) => {
            try {
                if (!Stripe)
                    throw new Error('Stripe not added');
                this.stripe = Stripe(secret);
            }
            catch (err) {
                if (retry > 0)
                    setTimeout(() => setup(retry--), 250);
                else
                    console.warn('Stripe failed, did you add the library & setup your API key?', err);
            }
        };
        setup();
    }
    async charge(amount, custom = {}) {
        this.emit('CHECKOUT', amount, custom);
        const request = await this.api.request({ url: '/api/payments', body: {
                amount,
                custom,
            } });
        return request.data.clientSecret;
    }
    async createForm(element, amount, custom) {
        const token = await this.charge(amount, custom);
        const form = this.stripe.elements({ clientSecret: token });
        form.create('payment').mount(element);
        return () => this.stripe.confirmPayment({
            elements: form,
            redirect: "if_required",
            confirmParams: { return_url: location.origin }
        });
    }
    async history(username) {
        const history = await this.api.request({ url: `/api/payments${username ? `/${username}` : ''}` });
        this.emit('LIST', username || null, history.data);
        return history.data;
    }
}
exports.Payments = Payments;
