"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pdf = void 0;
const api_1 = require("./api");
const utils_1 = require("@ztimson/utils");
class Pdf extends utils_1.TypedEmitter {
    api;
    constructor(api) {
        super();
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
    }
    async handleResponse(resp, fileName) {
        const blob = await resp.blob();
        if (fileName) {
            const url = URL.createObjectURL(blob);
            (0, utils_1.download)(url, fileName.endsWith('.pdf') ? fileName : fileName + '.pdf');
            URL.revokeObjectURL(url);
        }
        this.emit('CREATE', blob);
        return blob;
    }
    fromHtml(content, opts = {}) {
        return this.api.request({ url: `/api/pdf`, body: { html: content } })
            .then((resp) => this.handleResponse(resp, opts.download ? (opts.fileName || new Date().toISOString()) : undefined));
    }
    fromTemplate(template, data, opts = {}) {
        return this.api.request({ url: `/api/pdf${template}`, body: data })
            .then((resp) => this.handleResponse(resp, opts.download ? (opts.fileName || new Date().toISOString()) : undefined));
    }
    fromUrl(url, opts = {}) {
        return this.api.request({ url: `/api/pdf?url=${url}` }).then((resp) => this.handleResponse(resp, opts.download ? (opts.fileName || new Date().toISOString()) : undefined));
    }
}
exports.Pdf = Pdf;
