"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Settings = void 0;
const api_1 = require("./api");
const utils_1 = require("@ztimson/utils");
const rxjs_1 = require("rxjs");
class Settings extends utils_1.TypedEmitter {
    api;
    $cache = new rxjs_1.BehaviorSubject({});
    get cache() { return this.$cache.value; }
    set cache(val) { this.$cache.next(val); }
    constructor(api) {
        super();
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
    }
    list(detailed = false) {
        return this.api.request({ url: `/api/settings` + (detailed ? '?detailed' : '') }).then(resp => {
            this.cache = !detailed ? resp.data : Object.values(resp.data || {}).reduce((acc, v) => ({ ...acc, [v.key]: v.value }), {});
            this.emit('LIST', resp.data || []);
            return resp.data;
        });
    }
    delete(key) {
        return this.api.request({ url: `/api/settings/${key}`, method: 'DELETE' }).then(() => {
            this.cache = { ...this.cache, [key]: undefined };
            this.emit('DELETE', key);
        });
    }
    read(key, reload = false) {
        if (!reload && this.cache[key])
            return Promise.resolve(this.cache[key]);
        return this.api.request({ url: `/api/settings/${key}` }).then(variable => {
            if (variable.data)
                this.cache = { ...this.cache, [variable.data.key]: variable };
            this.emit('READ', variable.data);
            return variable.data;
        });
    }
    update(variable) {
        return this.api.request({ url: `/api/settings/${variable.key}`, body: variable }).then(variable => {
            if (variable.data)
                this.cache = { ...this.cache, [variable.data.key]: variable.data.value };
            this.emit('UPDATE', variable.data);
            return variable.data;
        });
    }
}
exports.Settings = Settings;
