"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Storage = void 0;
const api_1 = require("./api");
const utils_1 = require("@ztimson/utils");
class Storage extends utils_1.TypedEmitter {
    api;
    constructor(api) {
        super();
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
    }
    delete(path) {
        const url = (path.startsWith('/api/storage/') ? path : '/api/storage/' + path).replaceAll('//', '/');
        return this.api.request({ url, method: 'DELETE' }).then(() => {
            this.emit('DELETE', url);
        });
    }
    list(path) {
        const url = (path.startsWith('/api/storage/') ? path : '/api/storage/' + path).replaceAll('//', '/');
        return this.api.request({ url: url + '?list' }).then(resp => {
            this.emit('LIST', path, resp.data);
            return resp.data;
        });
    }
    open(path, target = '_blank') {
        const p = (path.startsWith('/api/storage/') ? path : '/api/storage/' + path).replaceAll(/\/{2,}/g, '/');
        const link = `${this.api.url}${p}${this.api.token ? `?token=${this.api.token}` : ''}`;
        if (!target)
            return link;
        this.emit('OPEN', path);
        return window.open(link, target);
    }
    mkdir(path) {
        const p = (path.startsWith('/api/storage/') ? path : '/api/storage/' + path).replaceAll(/\/{2,}/g, '/');
        return this.api.request({ url: p + '?directory', method: 'POST' }).then(resp => resp.data);
    }
    download(path, opts = {}) {
        const p = ('/api/storage/' + path).replaceAll('//', '/');
        return this.api.request({ ...opts, url: p, decode: false }).then(async (response) => {
            const blob = await response.data.blob();
            const name = opts.downloadAs || new URL(response.url).pathname.split('/').pop();
            this.emit('DOWNLOAD', path, blob);
            (0, utils_1.downloadBlob)(blob, name);
            return response.data;
        });
    }
    upload(files, opts = '') {
        return new utils_1.PromiseProgress(async (res, rej, prog) => {
            if (!files)
                files = await (0, utils_1.fileBrowser)();
            if (!files || Array.isArray(files) && !files.length)
                return [];
            const url = this.api.url + ('/api/storage/' + (typeof opts == 'string' ? opts : opts?.path)).replaceAll('//', '/');
            return (0, utils_1.uploadWithProgress)({
                url,
                files: Array.isArray(files) ? files : [files],
                headers: this.api.headers
            }).onProgress(p => {
                prog(p);
            }).then(resp => {
                this.emit('UPLOAD', resp);
                res(resp);
            }).catch(err => rej(err));
        });
    }
}
exports.Storage = Storage;
