"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Users = void 0;
const api_1 = require("./api");
const utils_1 = require("@ztimson/utils");
const rxjs_1 = require("rxjs");
class Users extends utils_1.TypedEmitter {
    api;
    listed = false;
    $cache = new rxjs_1.BehaviorSubject([]);
    get cache() { return this.$cache.value; }
    set cache(val) { this.$cache.next(val); }
    constructor(api) {
        super();
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
    }
    delete(username) {
        return this.api.request({
            url: `/api/users/${username}`,
            method: 'DELETE'
        }).then(() => {
            this.cache = this.cache.filter(u => u.username != username);
            this.emit('DELETE', username);
        });
    }
    list(reload = false) {
        if (!reload && this.listed)
            return Promise.resolve(this.cache);
        return this.api.request({ url: `/api/users` }).then(resp => {
            resp.data?.map(r => {
                r.image = this.api.url + r.image + `?token=${this.api.token}`;
                return r;
            });
            this.cache = resp.data || [];
            this.listed = true;
            this.emit('LIST', resp.data || []);
            return resp.data;
        });
    }
    read(username, reload = false) {
        if (!reload) {
            const cached = this.cache.find(u => u.username == username);
            if (cached)
                return Promise.resolve(cached);
        }
        return this.api.request({ url: `/api/users/${username}` }).then(resp => {
            if (resp.data) {
                resp.data.image = this.api.url + resp.data.image + `?token=${this.api.token}`;
                this.cache = [...this.cache.filter(u => u.username != username), resp.data];
            }
            this.emit('READ', resp.data);
            return resp.data;
        });
    }
    update(user) {
        return this.api.request({
            url: `/api/users/${user.username}`,
            method: 'PATCH',
            body: user
        }).then(resp => {
            if (resp.data) {
                resp.data.image = this.api.url + resp.data.image + `?token=${this.api.token}`;
                this.cache = this.cache.filter(u => u.username != user.username).concat([resp.data]);
            }
            this.emit(user._id ? 'UPDATE' : 'CREATE', resp.data);
            return resp.data;
        });
    }
    uploadImage(username, file) {
        return (0, utils_1.uploadWithProgress)({
            url: this.api.url + `/api/users/${username}/image`,
            files: [file],
            headers: this.api.headers,
        }).then(resp => {
            this.emit('UPLOAD_IMAGE', username, file);
            return resp;
        });
    }
}
exports.Users = Users;
